/*#define F_CPU 16000000UL */
#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_SET_LED    0
#define RQ_GET_SWITCH 1

#define RQ_SET_MAPX0 3
#define RQ_SET_MAPX2 4
#define RQ_SET_MAPX4 5
#define RQ_GET_SWITCH_CON 6

#define HasSWITCH
#ifdef HasSWITCH
	#define SwState2( p ) ( p ## s )
	#define SwitchOn( p ) ( ( PIND & (1<< port_ ## p ) ) == 0 )
	#define DoSwitch2( p ) { if( SwitchOn( p ) &&  SwState( p ) == 0 ) { \
									SwState( p ) = 1;\
									_delay_ms(30);\
								} else if( !SwitchOn(p) && SwState( p ) == 1 ) { \
									SwState( p ) = 0;\
									_delay_ms(30);\
								} }
	#define DoSwitch( p ) SwState(p) = SwitchOn( p )
	#define SwState( p ) SwitchOn( p )
#else
	//#define SwitchOn( p ) ( 0 )
	#define SwState( p ) ( 0 )
	#define DoSwitch( p ) {}
#endif

int map[6][6] = { {0,0,0,0,0,0} ,
		   {0,0,0,0,0,0} ,
		   {0,0,0,0,0,0} ,
		   {0,0,0,0,0,0} ,
		   {0,0,0,0,1,0} ,
		   {0,0,0,0,0,0} };

/*NUTJI.C*/
#define max_fre_two 5
int slot = 0;
int fre_two=0;
int two_status=0;
int us;

void scanLight()
{
	int i,mul;
	PORTB = 1<<slot;
	PORTC = 0;
	for(i=0,mul=32;i<6;i++,mul/=2)
	{
		if(map[5-i][5-slot]==0)
		{
			PORTC|=mul;
			//while(1);
		}
		if( map[5-i][5-slot] == 2 && us == 0 )
		{
			PORTC|=mul;
		}
		/*if(two_status==0&&map[slot][i]==2)
		{
			PORTC+=mul;
		}*/
	}
	//PORTB = 1<<3;
	//if( PORTC == 0 )
	//PORTC = 1 << slot | 1 << (slot+1);
	//else
	//PORTC = 1<<5;
	slot++;
	if(slot>=6)
	{
		slot=0;
		fre_two++;
		if(fre_two>=max_fre_two)
		{
			two_status=two_status==0?1:0;
			fre_two=0;
			us = (us+1)%2;
		}
	}
}

/*NUTJI.C*/

void setMap(int start,int b2_1,int b2_2)
{
	//return;
	int p,i;
	//p = 1<<5;
	p = 243;
	for(i=5;i>=0;i--)
	{ //111111
	  
		//if( p&b2_1 )
		if( b2_1%3 == 1 )
			map[start][i] = 1;//'#';
		else if( b2_1%3 == 2 )
			map[start][i] = 2;
		else 
			map[start][i] = 0;//'.';
		//p = p >> 1;
		b2_1/=3;
	}
	//p = 1<<5;
	p = 243;
	for(i=5;i>=0;i--)
	{
		if( b2_2%3 == 1)
			map[start+1][i] = 1;//'#';
		else if( b2_2%3 == 2 )
			map[start+1][i] = 2;
		else
			map[start+1][i] = 0;//'.';
		b2_2 /= 3;
		//p = p >>1;
	}
}

int ups,downs,lefts,rights;
int port_up,port_down,port_right,port_left;

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
	//return 0;
    usbRequest_t *rq = (void *)data;
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
	int i;
	//return 0;
	if (rq->bRequest == RQ_SET_MAPX0)
	{
		//PORTD = 0b00000000;
		
		setMap( 0 , rq->wIndex.word , rq->wValue.word );
		return 0;
	}else if (rq->bRequest == RQ_SET_MAPX2)
	{
		setMap( 2 , rq->wIndex.word , rq->wValue.word );
		return 0;
	}else if (rq->bRequest == RQ_SET_MAPX4)
	{
		setMap( 4 , rq->wIndex.word , rq->wValue.word );
		return 0;
	}else if( rq->bRequest == RQ_GET_SWITCH_CON)
	{
		//switch_state =  PIND & 0b01100011;
		//switch_state = PIND & (1<<PD0);
		//switch_state = SwitchOn( down );
		switch_state = SwState(up) | (SwState(down) << 1) | (SwState(left) << 2) | (SwState(right)<<3);
		usbMsgPtr = (uchar*) &switch_state;
        return sizeof(switch_state);
	}

    return 0;   /* default for not implemented requests: return no data back to host */
}


void InitSwitch()
{
	ups = 0;
	downs = 0;
	lefts = 0;
	rights = 0;
	
	port_up = PD6;
	port_down = PD0;
	port_left = PD5;
	port_right = PD1;
	
	DDRD &= ~( (1<<port_up) | (1<<port_down) | (1<<port_left) | (1<<port_right) );
	PORTD |= ( (1<<port_up) | (1<<port_down) | (1<<port_left) | (1<<port_right) );
}

void Init()
{
	int i,j;
	DDRC  = 0b00111111;  
	DDRB  = 0b00111111; 
//	PORTB = 0b00111111;
//	PORTC = 0b00000000;
//	while(1);
	usbInit();
	
	usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();
	
	sei();
	#ifdef HasSWITCH
		InitSwitch();
	#endif
	for( i = 0 ; i < 6 ; i++ )
		for( j = 0 ;j < 6 ; j++ )
			map[i][j] = 1;
	//map[2][3] = 1;
	//setMap( 0 , 63 , 53 );
}

//#undef HasSwitch

int main(void)
{
int i,j;
DDRD = 0b00001000;
		PORTD = 0b00000010;
Init();
while(1){
	usbPoll();
	_delay_ms(1);
	scanLight();
	if( PIND&(1<<PD1) == 0  )
	for( i = 0 ; i < 6 ; i++ )
		for( j=0;j<6;j++)
			map[i][j] = 0;
}
while(1);
//while(1);
	Init();
	//setMap( 0 , 63 , 53 );
    //DDRD &= 0b11111110;
//PORTD |= 1<<PD0;
//while(1);
    for(;;)
    {
        usbPoll();
		#ifdef HasSwitch
			/*DoSwitch( up );
			DoSwitch( down );
			DoSwitch( left );
			DoSwitch( right );*/
		#endif
	/*if(  PIND&(1<<PD5) == 0 ){
		PORTD = 0b00001000;
		_delay_ms(3000); 
	}*/
	
	//	PORTD = 0b00000000;
	if( PIND&(1<<PD1) == 1 ){
		PORTD = 0b00001000;
	}
	scanLight();
	_delay_ms( 1 );
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
